/*******************************************************
*   Inferno©
*   
*   This file may be used under the license as
*   stated below. For more information about License
*   types see the included LICENSE file. If you did 
*   not recive a copy of that file contact the author.
*   
*   @author  Frans van Nispen (xlr8@tref.nl)
*   @version beta 3
*   @date    December 2001
*   @license Inferno SDK Code
*
*******************************************************/
#ifndef _BITMAP_ACCESSOR_H
#define _BITMAP_ACCESSOR_H

#include <Bitmap.h>
#include <Menu.h>

#include "BitmapStructure.h"
#include "InfernoCommon.h"
#include "IColor.h"
#include "FilterTools.h"

#define WITHIN(a, b, c) ((((a) <= (b)) && ((b) <= (c))) ? 1 : 0)
#define ROUND(a)	( (a-long(a))>=.5)?(long(a)+1):(long(a))

#ifndef PI
	#define PI 3.14159265358979323846
#endif

// drawing modes
#define BRUSH_MODE		'brsM'
#define TRESHOLD_MODE	'trhM'

//common drawing modes
enum{
   BRUSH_NORMAL = 0,
   BRUSH_BLEND,
   BRUSH_DISSOLVE,
   BRUSH_SCREEN,
   BRUSH_OVER,
   BRUSH_LIGHTEN,
   BRUSH_DARKEN,
   BRUSH_DODGE,
   BRUSH_BURN,
   BRUSH_ADD,
   BRUSH_SUB,
   BRUSH_ERASE,
   BRUSH_PATTERN,
   BRUSH_DIFFERENCE,
   BRUSH_HUE,
   BRUSH_SATURATION,
   BRUSH_VALUE,
   BRUSH_SMUDGE,
   BRUSH_SMEAR,
   BRUSH_NONE,				// for non-filling of objects
   MODE_COUNT
};

//data structure to pass in all parameters to the drawfunctions
extern IBrush		*TBrush;				// must be set to costum or ICommon->GetBrush()
extern BBitmap		*TPattern;				// must be set to costum or ICommon->GetPattern()
extern rgb_color	TFg_color;
extern rgb_color	TBg_color;
extern int32		TMode;					// BRUSH_NORMAL bt default
extern int32		TTreshold_mode;			// TRESHOLD_RGB by default
extern int32		TTreshold;				// 0 by default
extern uint8		TPressure;				// 0-255 from tablet
extern bool			TAntialiase;			// True by default
extern int32		TFillMode;				// BRUSH_NONE by default (no fill)

extern uint32		BTchannel;				// needed for the LayerWindow

/*******************************************************
*   This is a list of helper function that let you 
*   easaly access the bitmapLayer. It does a lot of work
*   ajusting color spacess and accessing the correct
*   channel of the bitmap.  It also provides many optimized
*   methods for accessing the layer.  
*******************************************************/

/*******************************************************
* Initialise routines, needed for all add-ons that use
* these drawing tools
*******************************************************/

// This one should be called by the core upon layer changes &
// new or changed documents
void BitmapToolsInit(InfernoCommon *ICommon, bitmap_layer_info *bli, BBitmap *selection);

//Switch to a different bitmap_layer, can be restored too
void SetBitmapLayer(bitmap_layer_info *bli);
void SwitchBitmapLayer(bitmap_layer_info *bli);
void RestoreBitmapLayer();

/*******************************************************
* Get the selection value
*******************************************************/
uint8 GetSelect(BPoint p);
uint8 GetSelect(int32 x, int32 y);
BRect SelectionBounds();					// returns the bounds of a selection, nice to squizz undo

// This is for debugging only !!!!
void printRect(BRect r, const char *name = NULL);

/*******************************************************
* Pixel routines,
* Getting and setting a single pixle
* All pixel routines need manual selection checking if
* needed. ( Draw only if GetSelect != 0 )
*******************************************************/
//these routines draw to a layer or the current layer
rgb_color GetPixel(int32 x, int32 y);			// plain get pixel current layer
rgb_color GetPixel(BPoint p);
rgb_color GetBilinear(float x, float y);		// get with bilinear filtering
rgb_color GetBilinear(BPoint p);
void SetPixel(int32 x, int32 y, rgb_color c, bool useSelection = true);	// plain set pixel
void SetPixel(BPoint p, rgb_color c, bool useSelection = true);
void SetPixel(int32 x, int32 y, bool useSelection = true);				// plain set pixel, uses TFg_color
void SetPixel(BPoint p, bool useSelection = true);
void DrawPixel(int32 x, int32 y, bool useSelection = true);				// makes use of all draw modes
void DrawPixel(BPoint p, bool useSelection = true);

//these routines draw directly to a BBitmap
rgb_color GetRGB(BBitmap *bitmap, int32 x, int32 y);
rgb_color GetRGB(BBitmap *bitmap, BPoint);
rgb_color GetBilinearRGB(BBitmap *bitmap, float x, float y);		// get with bilinear filtering
rgb_color GetBilinearRGB(BBitmap *bitmap, BPoint p);
void PlotRGB(BBitmap *bitmap, int32 x, int32 y, rgb_color c);
void PlotRGB(BBitmap *bitmap, BPoint p, rgb_color c);


//these routines draw directly to a bitmap with swapping of Red and Blue for
//later access through a rgb_color* pointer
rgb_color GetBGR(BBitmap *bitmap, int32 x, int32 y);
rgb_color GetBGR(BBitmap *bitmap, BPoint);
void PlotBGR(BBitmap *bitmap, int32 x, int32 y, rgb_color c);
void PlotBGR(BBitmap *bitmap, BPoint p, rgb_color c);

/*******************************************************
* Brush routines
* Drawing a point with a brush, the brush is stored in
* data->brush
*******************************************************/
void BlitBrush(BPoint p, BBitmap *copy = NULL);
void BlitBrush(int32 x, int32 y, BBitmap *copy = NULL);
void BlitLine(BPoint start, BPoint end, BBitmap *copy = NULL);
void BlitLine(int32 x, int32 y, int32 xx, int32 yy, BBitmap *copy = NULL);

// if the TFillMode != BRUSH_NONE, the TBg_color will be used for filling
void BlitFrame(BRect r, BBitmap *copy = NULL);
void BlitFrame(BPoint start, BPoint end, BBitmap *copy = NULL);
void BlitFrame(int32 x, int32 y, int32 xx, int32 yy, BBitmap *copy = NULL);
void BlitEllipse(BRect r, BBitmap *copy = NULL);
void BlitEllipse(BPoint center, int32 rx, int32 ry, BBitmap *copy = NULL);
void BlitEllipse(int32 x, int32 y, int32 rx, int32 ry, BBitmap *copy = NULL);

/*******************************************************
* GUI routines, they need ICommon for the language file
* convversions (will be in the next version)
*******************************************************/
// exports a BMenu which gives a BRUSH_MODE message in which
// 'mode' gives a int32 of the chosen mode. This can be used
// to feed the draw routines
BMenu *DrawModeMenu(int32 mode = BRUSH_NORMAL, bool use_pattern = false, uint32 message = BRUSH_MODE);
// exports a BMenu which gives a TRESHOLD_MODE message in which
// 'mode' gives a int32 of the chosen mode. This can be used
// to feed the draw routines
BMenu *TresholdMenu(int32 mode = TRESHOLD_RGB, uint32 message = TRESHOLD_MODE);
// Draws a thumb on a view centered with a boundary square around it
void DrawThumb(BBitmap *thumb, BPoint p, BView *view, int32 T_SIZE = 150);
// Get a thumbnail of a bitmap with max dimentions of T_SIZE*T_SIZE
// The thumbnail is Bilinear filtered.
// If no bitmap is passed in, it will create a thumb of the current
// layer_bitmap.
BBitmap *GetThumb(int32 T_SIZE, BBitmap *input = NULL);
// draws a square layed in a view
void GUISquare(BRect r, BView *view);

/*******************************************************
* Math routines
*******************************************************/
float Distance(BPoint p1, BPoint p2);		// calculates the distance between two points
// if x<0 then x=0, if x>255 then x=255
uint8 Saturate(int x);
void SetRandomSeed(int32 seed);				// created a new randomTable with a new seed
int32 Random();								// Returns a new Random value
int32 Random(int32 x);						// Returns a new Random value
void SetRandom(int32 offset);				// can be used for stable random patterns
BRect CorrectRect(BRect r);					// makes sure the top-left is left on screen

/*******************************************************
* Clip handling.
* If the area is invalid, it returns NULL
*******************************************************/
BBitmap *CutBitmap(BBitmap *src, BRect area, rgb_color whipe_color);
BBitmap *CopyBitmap(BBitmap *src, BRect area);
void PasteBitmap(BBitmap *dest, BBitmap *clip, BPoint where);

#endif
