/*******************************************************
*   Inferno©
*   
*   This file may be used under the license as
*   stated below. For more information about License
*   types see the included LICENSE file. If you did 
*   not recive a copy of that file contact the author.
*   
*   @author  YNOP (ynop@acm.org)
*   @version 1.0
*   @date    Feb 5 2000
*   @license Inferno SDK Code
*******************************************************/
#ifndef _INFERNO_DOC_H
#define _INFERNO_DOC_H

#include <Bitmap.h>
#include <String.h>
#include <List.h>

#include "InfernoLayer.h"
class InfernoHistory;
class ScratchView;

#define I_DOC_CHANGED   'Info'
#define I_LAYER_CHANGED 'Ilcg'

enum layer_action{
   I_LAYER_ADD = 0,
   I_LAYER_DELETE,
   I_LAYER_SELECT,
   I_LAYER_SHOW_HIDE,
   I_LAYER_UPDATED,
   I_LAYER_OTHER
};

class InfernoDocument{
public:
   InfernoDocument(int32 W,int32 H);
   InfernoDocument(BMessage *prefs);
   void SaveState(BMessage *prefs);
   ~InfernoDocument();

   status_t AddLayer(InfernoLayer *layer, int32 index = -1); 
   status_t DeleteLayer(int32 index = -1); 
   InfernoLayer* RemoveLayer(int32 index = -1); 
   InfernoLayer* LayerAt(int32 index = -1);
   
   status_t SelectLayer(int32 index);
   int32 SelectedLayer() const;
   
   int32 CountLayers() const;
   
   void SetHidden(int32 index, bool ishidden);
   bool IsHidden(int32 index) const;
   
   BRect Bounds() const;
   
   status_t MergeLayers(int32 indexone,int32 indextwo);
   status_t Crop(int32 deltaLeft,int32 deltaTop,int32 deltaRight,int32 deltaBottom,bool croplayers = false);
   
   void EditLayerProperties(int32 index = -1);
   void EditImageProperties();
   
   status_t Lock();
   status_t Unlock();
public:
   bool Dirty;               // we are unclean (need to be saved)
   BBitmap *Selection;       // bitmap representing the selection
   BString ImageName;        // Name of the image
   BString Description;      // description of image
   ScratchView *scratchView; // the scratch view for this doc
   bool IsSelectionHidden;   // is the selec view hidden
   InfernoHistory *IHist;    // history for this doc
   BMessage *PrinterSettings;// BeOS Printer settings for this doc
private:
   InfernoDocument();
   InfernoDocument(const InfernoDocument&);
   InfernoDocument& operator=(InfernoDocument&);
private:
   BList layers;       // Our list of InfernoLayer
   int32 LayerCount;   // Total number of layers (should be layers.CountItems())
   int32 CurrentLayer; // Current Active layer
   int32 Width;        // Image Width
   int32 Height;       // Image Height
   sem_id DocSem;
};
#endif
