/*******************************************************
*   Inferno©
*   
*   This file may be used under the license as
*   stated below. For more information about License
*   types see the included LICENSE file. If you did 
*   not recive a copy of that file contact the author.
*   
*   @author  YNOP (ynop@acm.org)
*   @version 1.0
*   @date    Feb 5 2000
*   @license Inferno SDK Code
*******************************************************/
#ifndef INFERNO_HISTORY_H
#define INFERNO_HISTORY_H

class BString;
class BRect;
#include <Entry.h>

class InfernoDocument;
class InfernoLayer;

class YList;

struct history_lookup{
   int32 groupID;
   //BString name;
   off_t file_pos;
   ssize_t size;
};

#define I_RAW_UNDO 'Iraw'

/*******************************************************
*   
*******************************************************/
class InfernoHistory{
public:
   InfernoHistory(InfernoDocument *doc);
   ~InfernoHistory();
   
   // Begins a history group so that all itmes
   // added durning this session can be undone
   // at once.
   void BeginHistoryGroup(const char *name);
   status_t EndHistoryGroup();
   
   // this is all a disk based save.  It would be nice
   // if we could somehowe put some of this into a 
   // small mem buffer so the most current ones 
   // can be accessed quickly.
   
   // Basic save data from a layer, pass the layer pointer
   // and the undo rect area. 
   status_t Save(InfernoLayer *ilay,BRect undoRect);
   
   // Saves a raw msg you packed up yourself.  If you want
   // you can give it a name. Each Raw msg assumes its own 
   // group ID. Thus they can not be grouped together.
   status_t SaveRaw(BMessage *rawmsg,BLooper *target,const char *historyName = NULL);
   
   // Saves off the selection map to the undo buffer
   status_t SaveSelection();
   
   // Resotors from History the last group/action and
   // udates the IDoc
   status_t Restore();
   
   // Redos the last action affected by Restore
   status_t Redo();
   
   // Branch causes the current document to 
   // save off to disk along with its undo history
   // and back out all of the undos to that point.
   // then creat a new document and start over 
   status_t Branch(const char *branchname, int32 i);
   status_t Merge(const char *branchname);
   
   
   // Hooks needed to wirte a History window plugin
   // it should be ablet to list all the history items
   // by there group name and also be able to expand 
   // each group into its individual items.
   // Also you should be able to back out several
   // undo groups at once ... this will require
   // lots of work :(
   
  // BList* GetBranchList();
  // BList* GetUndoList();
  
  // Get the groups in the stack. Undo is a stack
  // So the item at the end is newest item, and item 
  // at Zero is the oldest.
  
  // int32 CountGroups();
  
  // Returns the name that was set when this group got 
  // created.
  // const char* GroupName(int32 i);
  
  // Adds a hook to a single group. When that group gets pass
  // out for undo this hook will be called on the outgouing
  // message.
  // status_t AddGroupHook(int32 i,bool (*func)(BMessage*));
  // status_t RemoveGroupHook(int32 i);
  
  // this sets how many items (in group terms)
  // that the undo stack should keep, note that
  // this must be in undo terms as each tool can
  // cause many many undo actions.  Passing in a 
  // -1 will cause a infinate stack size.
  // Resizeing down will chop off old undos btw.
  
  //void SetUndoSize(int32 s = -1);
  
  // Flushes the undo stack back count items.
  // -1 wil cause a full flush of the entire stack
  //void FlushUndo(int32 count = -1);
  
  // Writes any buffered data into the undofile.
  // void Sync();
  
private:
   status_t DumpToFile(BMessage*);
   void HandleRaw(BMessage *msg);
private:
   InfernoHistory();
   InfernoHistory(const InfernoHistory&);
   InfernoHistory& operator=(InfernoHistory&);
private:
   InfernoDocument *IDoc;
   BFile *HistoryFile;
   entry_ref file_ref;
   YList *hist;
   uint32 curID;
   bool grouping;
   
   sem_id histSem;
   
   //BOutlineListView *ollv;
};
#endif
