/*******************************************************
*   Inferno©
*   
*   This file may be used under the license as
*   stated below. For more information about License
*   types see the included LICENSE file. If you did 
*   not recive a copy of that file contact the author.
*   
*   @author  Frans van Nispen (xlr8@tref.nl)
*   @version 1.0
*   @date    June 2001
*   @license Inferno SDK Code
*
*	There is a lot of code in here, it might be needed
*	to split some stuff up in different headers.
*******************************************************/
#ifndef _I_COLOR_H
#define _I_COLOR_H

//common ways to equate two pixels
enum{
	TRESHOLD_RGB,
	TRESHOLD_HSV,
	TRESHOLD_HUE,
	TRESHOLD_LUM
};

/*******************************************************
* HSV
*
* Hue 0.0 - 360.0
* Saturation 0.0 - 1.0
* Value 0.0 - 1.0
*******************************************************/
typedef struct hsv_color{
	double hue;
	double saturation; 
	double value;
	uint8 alpha;
} hsv_color;

/*******************************************************
* HLS
*
* Hue 0.0 - 360.0
* Luminance 0.0 - 1.0
* Saturation 0.0 - 1.0
*******************************************************/
typedef struct hls_color{
	double hue;
	double luminance;
	double saturation; 
	uint8 alpha;
} hls_color;

/*******************************************************
* CMY
*
* Cyan 0 - 255
* Maganta 0 - 255
* Yellow 0 - 255
*******************************************************/
typedef struct cmy_color{
	uint8 cyan;
	uint8 magenta; 
	uint8 yellow;
	uint8 alpha;
} cmy_color;

/*******************************************************
* CMY
*
* Cyan 0 - 255
* Maganta 0 - 255
* Yellow 0 - 255
* BlacK 0 - 255;
*******************************************************/
typedef struct cmyk_color{
	uint8 cyan;
	uint8 magenta; 
	uint8 yellow;
	uint8 black;
	uint8 alpha;
} cmyk_color;



/*******************************************************
* Color Conversion & checking routines
*******************************************************/
// conversion routines
rgb_color HSVtoRGB(hsv_color in);
hsv_color RGBtoHSV(rgb_color in);

rgb_color HLStoRGB(hls_color in);
hls_color RGBtoHLS(rgb_color in);

rgb_color CMYtoRGB(cmy_color in);
cmy_color RGBtoCMY(rgb_color in);

rgb_color CMYKtoRGB(cmyk_color in);
cmyk_color RGBtoCMYK(rgb_color in);

uint8 RGBtoGray(rgb_color in);

// converts an rgb_color to bgr so it can be put directly in a bitmap
// with a 32bit move instruction. This can speed things up dramaticly.
rgb_color RGBtoBGR(rgb_color in);

// routines to compair two pixels to match in color with a treshold
bool CheckRGB(rgb_color src, rgb_color dest, int32 treshold);
bool CheckHSV(rgb_color src, rgb_color dest, int32 treshold);
bool CheckLuminance(rgb_color src, rgb_color dest, int32 treshold);
bool CheckHue(rgb_color src, rgb_color dest, int32 treshold);
bool CheckColors(rgb_color src, rgb_color dest, int32 treshold, int32 mode);

#endif
