/*******************************************************
*   Inferno©
*   
*   This file may be used under the license as
*   stated below. For more information about License
*   types see the included LICENSE file. If you did 
*   not recive a copy of that file contact the author.
*   
*   @author  YNOP (ynop@acm.org)
*   @version 1.0
*   @date    Feb 5 2000
*   @license Inferno SDK code
*******************************************************/
#ifndef INFERNO_FILTER_H
#define INFERNO_FILTER_H

#ifdef BUILDING_I_CORE
#define FILTER_SYMBOLS _IMPORT
#else
#define FILTER_SYMBOLS _EXPORT
#endif

#include "InfernoDocument.h"
#include "InfernoCommon.h"

// When running Filter you will get passed a quality
// to render at... The two you should focus on is 
// HIGH_QUALITY as this is what the final filter will
// render at. And LOW_QUALITY as this is what will be
// used to create a Proof.
enum proof_quality{
  I_PROOF_LOW_QUALITY = 0,
  I_PROOF_HIGH_QUALITY = 5
};

// All the info about how to treat selections
// and the bounding box
struct filter_mode{
   BRect BoundingBox;
   BBitmap *selection;
   bool useSelection;
};

#define COLOR_CATAGORY    "FILTER_COLOR_CATAGORY"    // Oh the pritty colors
#define IMAGE_CATAGORY    "FILTER_IMAGE_CATAGORY"    // 
#define GENERIC_CATAGORY  "FILTER_GENERIC_CATAGORY"  // Duh, if you just dont know
#define BLUR_CATAGORY     "FILTER_BLUR_CATAGORY"     // Bbbbllureee.....
#define SHARPEN_CATAGORY  "FILTER_SHARPEN_CATAGORY"  // SHARP
#define EFFECT_CATAGORY   "FILTER_EFFECTS_CATAGORY"  // 3ph3kz
#define DISTORT_CATAGORY  "FILTER_DISTORT_CATAGORY"  // DiStOrTiOnS
#define ANALYZER_CATAGORY "FILTER_ANALYZER_CATAGORY" //
#define NOISE_CATAGORY    "FILTER_NOISE_CATAGORY"    // N.%$^O%^.@!(I%,.S#$*^E
#define RENDER_CATAGORY   "FILTER_RENDER_CATAGORY"   //
#define TEXTURE_CATAGORY  "FILTER_TEXTURE_CATAGORY"  //
#define SKETCH_CATAGORY   "FILTER_SKETCH_CATAGORY"   //
#define VIDEO_CATAGORY    "FILTER_VIDEO_CATAGORY"    //
#define STYLIZE_CATAGORY  "FILTER_STYLIZE_CATAGORY"  // 
#define OTHER_CATAGORY    "FILTER_OTHER_CATAGORY"    // 

#define INFERNO_FILTER_POSITION_ATTR "Inferno:FilterPosition" // uint32 attribut discribing sub menu user position

#define FILTER_API_VER LAYER_API_VER+001 // @skip

class FILTER_SYMBOLS InfernoFilter {
public:
   InfernoFilter(BMessage *prefs,InfernoCommon *IC,bool CanDoProof = false);
   virtual ~InfernoFilter();
   virtual void SaveState(BMessage *prefs);
   virtual BView* GetSettingsView();
   virtual status_t Filter(InfernoLayer *ILay, filter_mode fm, proof_quality quality) = 0;
   status_t Proof(proof_quality quality = I_PROOF_LOW_QUALITY);
   BBitmap* Thumb(int32 size);
   
   virtual status_t Perform(perform_code d, void *arg);
protected:
   InfernoCommon *ICommon;
private:
   virtual void _ReservedFilter1();
   virtual void _ReservedFilter2();
   
   InfernoFilter();
   InfernoFilter(const InfernoFilter&);
   InfernoFilter& operator=(InfernoFilter&);
private:
   bool cdp;
   
   uint32 _reserved[2];
};
extern "C" FILTER_SYMBOLS InfernoFilter *instantiate_filter(BMessage *prefs,InfernoCommon *IC);
#endif




