/*******************************************************
*   Inferno©
*   
*   This file may be used under the license as
*   stated below. For more information about License
*   types see the included LICENSE file. If you did 
*   not recive a copy of that file contact the author.
*   
*   @author  YNOP (ynop@acm.org)
*   @version beta
*   @date    Feb 5 2000
*   @license Inferno SDK code
*******************************************************/
#ifndef INFERNO_LAYER_H
#define INFERNO_LAYER_H

#ifdef BUILDING_I_CORE
#define LAYER_SYMBOLS _IMPORT
#else
#define LAYER_SYMBOLS _EXPORT
#endif

#include <String.h>
#include <Point.h>
#include <Rect.h>
#include <Bitmap.h>
#include <View.h>

class InfernoCommon;
class InfernoDocument;
class InfernoHistory;

#define LAYER_API_VER 007 // @skip

#define LAYER_BOUNDS_BUG 1 // if true then the Layer bounds bug has not been fixed.

#define THUMB_NAIL_SIZE 64 // Default thumb size

/*******************************************************

*******************************************************/
class LAYER_SYMBOLS InfernoLayer{
friend InfernoHistory;
friend InfernoDocument;
public:
   InfernoLayer(const BMessage *initmsg,InfernoCommon *IC);
   virtual ~InfernoLayer();
   virtual status_t Init(const BMessage *initmsg);
   virtual void SaveState(BMessage *prefs);
   virtual status_t InitCheck();
   
   virtual BRect Bounds() const;
   
   virtual void* GetData() = 0;
   virtual status_t Render(BRect area,BBitmap *final,BView *view) = 0;
   
   virtual BBitmap* Thumb(int32 size = THUMB_NAIL_SIZE);
   virtual BView* GetLayerConfig();
   
   virtual void Cut(BMessage *clip,BBitmap *selection);
   virtual void Copy(BMessage *clip,BBitmap *selection);
   virtual void Paste(BMessage *clip);
   
   virtual status_t Crop(int32 deltaLeft,int32 deltaTop,int32 deltaRight,int32 deltaBottom);
   
   bool IsHidden() const;
   
   void MoveTo(BPoint where);
   void MoveTo(float x, float y);
   BPoint Position() const;
   
   const char* GetType() const;
   const char* GetEditTool() const;
   
   const char* Name() const;
   uint8 Alpha() const;
   void SetName(const char *name);
   void SetAlpha(uint8 alpha);
   
   virtual status_t Perform(perform_code d, void *arg);
protected:
   void SetHidden(bool ishidden);
   virtual status_t SaveUndoState(BRect undoRect, BMessage *msg/*,BMessage *extradata = NULL*/) = 0;
   virtual status_t ExecuteUndo(BMessage *msg) = 0;
   //virtual void GetSuportedBlockSizes(BRect **rects,int32 *count);
protected:
   InfernoCommon *ICommon;
   status_t layer_status;
   BString LayerType;
   BString EditToolName; // Not used for much right now
   int32 H;
   int32 W;
   BPoint Pos;
   bool Hidden;
private:
   virtual void _ReservedLayer1();
   virtual void _ReservedLayer2();

   InfernoLayer();
   InfernoLayer(const InfernoLayer&);
   InfernoLayer &operator=(InfernoLayer&);

   BString LayerName;
   uint8 LayerAlpha; 
   
   uint32 _reserved[4];
};
extern "C" LAYER_SYMBOLS InfernoLayer *instantiate_layer(BMessage*,InfernoCommon*);
#endif
